@@ -11,7 +11,7 @@ from emq.range.constants import GALAXY_EMQ_QUEUE_DELAY_SECONDS_MINIMAL, GALAXY_E
   GALAXY_EMQ_MESSAGE_DELAY_SECONDS_MAXIMAL, GALAXY_EMQ_MESSAGE_DELAY_SECONDS_MINIMAL, \
   GALAXY_EMQ_MESSAGE_INVISIBILITY_SECONDS_MAXIMAL, GALAXY_EMQ_MESSAGE_INVISIBILITY_SECONDS_MINIMAL, \
   GALAXY_EMQ_QUEUE_WRITE_QPS_MINIMAL, GALAXY_EMQ_QUEUE_WRITE_QPS_MAXIMAL, GALAXY_EMQ_QUEUE_READ_QPS_MINIMAL, \
-  GALAXY_EMQ_QUEUE_READ_QPS_MAXIMAL, GALAXY_EMQ_QUEUE_MAX_SPACE_QUOTA_MINIMAL, GALAXY_EMQ_QUEUE_MAX_SPACE_QUOTA_MAXIMAL, \
+  GALAXY_EMQ_QUEUE_READ_QPS_MAXIMAL, \
   GALAXY_EMQ_QUEUE_REDRIVE_POLICY_MAX_RECEIVE_TIME_MINIMAL, GALAXY_EMQ_QUEUE_REDRIVE_POLICY_MAX_RECEIVE_TIME_MAXIMAL
 from emq.message.ttypes import SendMessageRequest, ReceiveMessageRequest, ChangeMessageVisibilityRequest, \
   DeleteMessageRequest, SendMessageBatchRequest, SendMessageBatchRequestEntry, ChangeMessageVisibilityBatchRequestEntry, \
@@ -19,7 +19,7 @@ from emq.message.ttypes import SendMessageRequest, ReceiveMessageRequest, Change
 from emq.queue.ttypes import CreateQueueRequest, ListQueueRequest, SetQueueAttributesRequest, SetPermissionRequest, \
   RevokePermissionRequest, QueryPermissionForIdRequest, SetQueueQuotaRequest, QueueQuota, CreateTagRequest, \
   DeleteTagRequest, GetTagInfoRequest, ListTagRequest, SetQueueRedrivePolicyRequest, RemoveQueueRedrivePolicyRequest, \
-  ListDeadLetterSourceQueuesRequest
+  CopyQueueRequest, QueryPrivilegedQueueRequest, DeleteQueueRequest
 from emq.statistics.ttypes import SetUserQuotaRequest, GetUserQuotaRequest, GetUserUsedQuotaRequest, SetUserInfoRequest, \
   GetUserInfoRequest
 
@@ -32,14 +32,15 @@ class RequestChecker(object):
     if len(self.__args) > 1:
       errMsg = "Unknown request"
       raise GalaxyEmqServiceException(errMsg=errMsg)
-    else:
+    elif len(self.__args) == 1:
       self.check_request_params(self.__args[0])
 
   def check_request_params(self, request):
-    if isinstance(request, ListQueueRequest):
+    if isinstance(request, ListQueueRequest) or \
+        isinstance(request, QueryPrivilegedQueueRequest):
       self.validate_queue_prefix(request.queueNamePrefix)
     elif isinstance(request, CreateQueueRequest):
-      self.validate_queue_name(request.queueName, False)
+      self.validate_queue_name(request.queueName, True, True)
       self.validate_queue_attribute(request.queueAttribute)
       self.validate_queue_quota(request.queueQuota)
     elif isinstance(request, SetQueueAttributesRequest):
@@ -53,8 +54,6 @@ class RequestChecker(object):
       self.validate_redrivePolcy(request.redrivePolicy)
     elif isinstance(request, RemoveQueueRedrivePolicyRequest):
       self.validate_queue_name(request.queueName)
-    elif isinstance(request, ListDeadLetterSourceQueuesRequest):
-      self.validate_queue_name(request.dlqName)
     elif isinstance(request, SetPermissionRequest):
       self.validate_queue_name(request.queueName)
       self.validate_not_none(request.developerId, "developerId")
@@ -166,6 +165,13 @@ class RequestChecker(object):
           or isinstance(request, GetUserUsedQuotaRequest) or isinstance(request, SetUserInfoRequest)
           or isinstance(request, GetUserInfoRequest)):
       pass
+    elif isinstance(request, DeleteQueueRequest):
+      self.validate_queue_name(request.queueName)
+    elif isinstance(request, CopyQueueRequest):
+      queueMeta = request.queueMeta
+      self.validate_queue_name(queueMeta.queueName)
+    elif isinstance(request, basestring):
+      self.validate_queue_name(request)
     else:
       self.validate_queue_name(request.queueName)
 
@@ -336,12 +342,6 @@ class RequestChecker(object):
                            GALAXY_EMQ_MESSAGE_INVISIBILITY_SECONDS_MAXIMAL,
                            "messageInvisibilitySeconds")
 
-  def validate_spaceQuota(self, spaceQuota):
-    self.check_filed_range(spaceQuota,
-                           GALAXY_EMQ_QUEUE_MAX_SPACE_QUOTA_MINIMAL,
-                           GALAXY_EMQ_QUEUE_MAX_SPACE_QUOTA_MAXIMAL,
-                           "spaceQuota")
-
   def validate_readQps(self, readQps):
     self.check_filed_range(readQps,
                            GALAXY_EMQ_QUEUE_READ_QPS_MINIMAL,
