@@ -17,9 +17,10 @@
 # specific language governing permissions and limitations
 # under the License.
 #
-
+import base64
 import httplib
 import os
+import rfc822
 import socket
 import sys
 import urllib
@@ -30,18 +31,18 @@ import hmac
 from cStringIO import StringIO
 
 from thrift.transport.TTransport import TTransportBase
-from thrift.transport.TTransport import TMemoryBuffer
-from thrift.protocol.TJSONProtocol import TJSONProtocol
 from sds.auth.constants import HK_TIMESTAMP
 from sds.auth.constants import HK_HOST
 from sds.auth.constants import HK_CONTENT_MD5
 from sds.auth.constants import HK_AUTHORIZATION
-from sds.auth.constants import HttpAuthorizationHeader
-from sds.auth.constants import MacAlgorithm
+from sds.auth.constants import MI_DATE
+from sds.auth.constants import XIAOMI_HEADER_PREFIX
 from sds.errors.constants import HttpStatusCode
 from sds.client.exceptions import SdsTransportException
 from sds.common.ttypes import ThriftProtocol
 from sds.common.constants import THRIFT_HEADER_MAP
+from urlparse import urlparse
+from hashlib import sha1
 
 
 class SdsTHttpClient(TTransportBase):
@@ -49,7 +50,7 @@ class SdsTHttpClient(TTransportBase):
 
   def __init__(self, credential, uri_or_host, timeout=None, thrift_protocol=ThriftProtocol.TBINARY):
     self.credential = credential
-    parsed = urlparse.urlparse(uri_or_host)
+    parsed = urlparse(uri_or_host)
     self.scheme = parsed.scheme
     assert self.scheme in ('http', 'https')
     if self.scheme == 'http':
@@ -121,10 +122,7 @@ class SdsTHttpClient(TTransportBase):
     self.__http.putrequest('POST', self.path)
 
     # Write headers
-    self.__http.putheader('Host', self.host)
-    self.__http.putheader('User-Agent', 'Python/SdsTHttpClient')
-    self.__http.putheader('Content-Type', THRIFT_HEADER_MAP[self.__protocol])
-    self.__http.putheader('Content-Length', str(len(data)))
+    headers = self.__set_headers(data)
 
     if not self.__custom_headers or 'User-Agent' not in self.__custom_headers:
       user_agent = 'Python/THttpClient'
@@ -133,11 +131,7 @@ class SdsTHttpClient(TTransportBase):
         user_agent = '%s (%s)' % (user_agent, urllib.quote(script))
       self.__http.putheader('User-Agent', user_agent)
 
-    if self.__custom_headers:
-      for key, val in self.__custom_headers.iteritems():
-        self.__http.putheader(key, val)
-
-    for key, val in self.__auth_headers(data).iteritems():
+    for key, val in self.__auth_headers(dict(headers.items() + self.__custom_headers.items())).iteritems():
       self.__http.putheader(key, val)
 
     self.__http.endheaders()
@@ -158,25 +152,96 @@ class SdsTHttpClient(TTransportBase):
   if hasattr(socket, 'getdefaulttimeout'):
     flush = __withTimeout(flush)
 
-  def __auth_headers(self, body):
-    headers = dict()
-    headers[HK_HOST] = self.host
-    headers[HK_TIMESTAMP] = str(int(time.time() + self.__clock_offset))
-    headers[HK_CONTENT_MD5] = hashlib.md5(body).hexdigest()
-
-    auth_header = HttpAuthorizationHeader()
-    auth_header.algorithm = MacAlgorithm.HmacSHA1
-    auth_header.userType = self.credential.type
-    auth_header.secretKeyId = self.credential.secretKeyId
-
-    auth_header.signedHeaders = list(headers.iterkeys())
-    buf = "\n".join([headers[x] for x in auth_header.signedHeaders])
-    auth_header.signature = \
-      hmac.new(self.credential.secretKey, buf, hashlib.sha1).hexdigest()
-
-    mb = TMemoryBuffer()
-    protocol = TJSONProtocol(mb)
-    auth_header.write(protocol)
-    headers[HK_AUTHORIZATION] = str(mb.getvalue())
+  def __auth_headers(self, headers):
+    string_to_assign = str()
+    string_to_assign += '%s\n' % 'POST'
+    string_to_assign += '%s\n' % self.__get_header(headers, "content-md5")
+    string_to_assign += '%s\n' % self.__get_header(headers, "content-type")
+    string_to_assign += '\n'
+    string_to_assign += '%s' % self.__canonicalize_xiaomi_headers(headers)
+    string_to_assign += '%s' % self.__canonicalize_resource(self.path)
+    signature = \
+      base64.encodestring(hmac.new(self.credential.secretKey, string_to_assign, digestmod=sha1).digest()).strip()
+    auth_string = "Galaxy-V3 %s:%s" % (self.credential.secretKeyId, signature)
+    headers[HK_AUTHORIZATION] = auth_string
 
     return headers
+
+  def __set_headers(self, body):
+    headers = dict()
+    headers[HK_HOST] = self.host
+    headers['content-length'] = str(len(body))
+    headers[HK_TIMESTAMP] = str(int(time.time() + self.__clock_offset))
+    headers[HK_CONTENT_MD5] = hashlib.md5(body).hexdigest()
+    headers['content-type'] = THRIFT_HEADER_MAP[self.__protocol]
+    headers[MI_DATE] = rfc822.formatdate(time.time())
+    return headers
+
+  def __get_header(self, http_headers, header_name):
+    if http_headers is None or len(http_headers) == 0:
+      return ''
+    for key in http_headers:
+      lower_key = key.lower()
+      try:
+        lower_key = lower_key.decode('utf-8')
+      except:
+        pass
+      if lower_key == header_name and http_headers[key]:
+        if type(http_headers[key]) != str:
+          return http_headers[key][0]
+        else:
+          return http_headers[key]
+    return ''
+
+
+  def __canonicalize_xiaomi_headers(self, http_headers):
+    if http_headers is None or len(http_headers) == 0:
+      return ''
+
+    canonicalized_headers = dict()
+    for key in http_headers:
+      lower_key = key.lower()
+      try:
+        lower_key = lower_key.decode('utf-8')
+      except:
+        pass
+
+      if http_headers[key] and lower_key.startswith(XIAOMI_HEADER_PREFIX):
+        if type(http_headers[key]) != str:
+          canonicalized_headers[lower_key] = str()
+          i = 0
+          for k in http_headers[key]:
+            canonicalized_headers[lower_key] += '%s' % (k.strip())
+            i += 1
+            if i < len(http_headers[key]):
+              canonicalized_headers[lower_key] += ','
+        else:
+          canonicalized_headers[lower_key] = http_headers[key].strip()
+
+    result = ""
+    for key in sorted(canonicalized_headers.keys()):
+      values = canonicalized_headers[key]
+      result += '%s:%s\n' % (key, values)
+    return result
+
+  def __canonicalize_resource(self, uri):
+    result = ""
+    parsed_url = urlparse(uri)
+    result += '%s' % parsed_url.path
+    query_args = parsed_url.query.split('&')
+    subresource = ['acl', 'quota', 'uploads', 'partNumber', 'uploadId', 'storageAccessToken', 'metadata']
+
+    i = 0
+    for query in sorted(query_args):
+      key = query.split('=')
+      if key[0] in subresource:
+        if i == 0:
+          result += '?'
+        else:
+          result += '&'
+        if len(key) == 1:
+          result += '%s' % key[0]
+        else:
+          result += '%s=%s' % (key[0], key[1])
+        i += 1
+    return result
