@@ -12,11 +12,9 @@ def datum_scan_iter(table_client, scan):
       # throttling the request
       sleep(0.5 * (1 << (retry - 1)))
     result = table_client.scan(scan)
-    terminated = result.nextStartKey is None and result.nextSplitIndex == -1
+    terminated = result.nextStartKey is None
     records = result.records
     scan.startKey = result.nextStartKey
-    if result.nextSplitIndex > 0:
-      scan.splitIndex = result.nextSplitIndex
     if not terminated and len(records) < scan.limit and result.throttled:
       retry += 1
     else:
